@@ -4,12 +4,14 @@
  *   SPDX-FileCopyrightText: 2008-2011 Matthias Fuchs <mat69@gmx.net>
  *   SPDX-FileCopyrightText: 2012 Reza Fatahilah Shah <rshah0385@kireihana.com>
  *   SPDX-FileCopyrightText: 2015 Marco Martin <mart@kde.org>
+ *   SPDX-FileCopyrightText: 2022 Alexander Lohnau <alexander.lohnau@gmx.de>
  *
  *   SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "comic.h"
 #include "checknewstrips.h"
+#include "comic_debug.h"
 #include "comicarchivedialog.h"
 #include "comicarchivejob.h"
 #include "comicsaver.h"
@@ -17,6 +19,9 @@
 
 #include <QAction>
 #include <QDebug>
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+#include <QNetworkInformation>
+#endif
 #include <QScreen>
 #include <QSortFilterProxyModel>
 #include <QTimer>
@@ -30,18 +35,14 @@
 #include <kuiserverjobtracker.h>
 
 #include <Plasma/Containment>
-#include <Plasma/Package>
-#include <Plasma/PluginLoader>
 
 #include "comicmodel.h"
 #include "comicupdater.h"
 
 Q_GLOBAL_STATIC(ComicUpdater, globalComicUpdater)
 
-const int ComicApplet::CACHE_LIMIT = 20;
-
-ComicApplet::ComicApplet(QObject *parent, const QVariantList &args)
-    : Plasma::Applet(parent, args)
+ComicApplet::ComicApplet(QObject *parent, const KPluginMetaData &data, const QVariantList &args)
+    : Plasma::Applet(parent, data, args)
     , mProxy(nullptr)
     , mActiveComicModel(new ActiveComicModel(parent))
     , mDifferentComic(true)
@@ -56,10 +57,11 @@ ComicApplet::ComicApplet(QObject *parent, const QVariantList &args)
     , mMaxComicLimit(0)
     , mCheckNewStrips(nullptr)
     , mActionShop(nullptr)
-    , mEngine(nullptr)
+    , mEngine(new ComicEngine(this))
     , mSavingDir(nullptr)
 {
     setHasConfigurationInterface(true);
+    connect(mEngine, &ComicEngine::requestFinished, this, &ComicApplet::dataUpdated);
 }
 
 void ComicApplet::init()
@@ -69,8 +71,7 @@ void ComicApplet::init()
 
     configChanged();
 
-    mEngine = dataEngine(QStringLiteral("comic"));
-    mModel = new ComicModel(mEngine, QStringLiteral("providers"), mTabIdentifier, this);
+    mModel = new ComicModel(mEngine, mTabIdentifier, this);
     mProxy = new QSortFilterProxyModel(this);
     mProxy->setSourceModel(mModel);
     mProxy->setSortCaseSensitivity(Qt::CaseInsensitive);
@@ -133,6 +134,28 @@ void ComicApplet::init()
 
     updateUsedComics();
     changeComic(true);
+
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+    QT_WARNING_PUSH
+    QT_WARNING_DISABLE_DEPRECATED
+    connect(&m_networkConfigurationManager, &QNetworkConfigurationManager::onlineStateChanged, this, [this](bool isOnline) {
+        if (!isOnline) {
+            return;
+        }
+        qCDebug(PLASMA_COMIC) << "Online status changed to true, requesting comic" << mPreviousFailedIdentifier;
+        mEngine->requestSource(mPreviousFailedIdentifier);
+    });
+    QT_WARNING_POP
+#else
+    QNetworkInformation::instance()->load(QNetworkInformation::Feature::Reachability);
+    connect(QNetworkInformation::instance(), &QNetworkInformation::reachabilityChanged, this, [this](auto reachability) {
+        if (reachability != QNetworkInformation::Reachability::Online) {
+            return;
+        }
+        qCDebug(PLASMA_COMIC) << "Online status changed to true, requesting comic" << mPreviousFailedIdentifier;
+        mEngine->requestSource(mPreviousFailedIdentifier);
+    });
+#endif
 }
 
 ComicApplet::~ComicApplet()
@@ -140,26 +163,23 @@ ComicApplet::~ComicApplet()
     delete mSavingDir;
 }
 
-void ComicApplet::dataUpdated(const QString &source, const Plasma::DataEngine::Data &data)
+void ComicApplet::dataUpdated(const ComicMetaData &data)
 {
+    const QString source = data.identifier;
     setBusy(false);
 
     // disconnect prefetched comic strips
-    if (mEngine && source != mOldSource) {
-        mEngine->disconnectSource(source, this);
+    if (source != mOldSource) {
         return;
     }
 
     setConfigurationRequired(false);
 
     // there was an error, display information as image
-    const bool hasError = data[QStringLiteral("Error")].toBool();
-    const bool errorAutoFixable = data[QStringLiteral("Error automatically fixable")].toBool();
-    if (hasError) {
-        const QString previousIdentifierSuffix = data[QStringLiteral("Previous identifier suffix")].toString();
-        if (mEngine && !mShowErrorPicture && !previousIdentifierSuffix.isEmpty()) {
-            mEngine->disconnectSource(source, this);
-            updateComic(previousIdentifierSuffix);
+    if (data.error) {
+        mPreviousFailedIdentifier = source;
+        if (!mShowErrorPicture && !data.previousIdentifier.isEmpty()) {
+            updateComic(data.previousIdentifier);
         }
         return;
     }
@@ -178,21 +198,14 @@ void ComicApplet::dataUpdated(const QString &source, const Plasma::DataEngine::D
     // call the slot to check if the position needs to be saved
     slotStorePosition();
 
-    if (mEngine) {
-        // disconnect if there is either no error, or an error that can not be fixed automatically
-        if (!errorAutoFixable) {
-            mEngine->disconnectSource(source, this);
-        }
-
-        // prefetch the previous and following comic for faster navigation
-        if (mCurrent.hasNext()) {
-            const QString prefetch = mCurrent.id() + QLatin1Char(':') + mCurrent.next();
-            mEngine->connectSource(prefetch, this);
-        }
-        if (mCurrent.hasPrev()) {
-            const QString prefetch = mCurrent.id() + QLatin1Char(':') + mCurrent.prev();
-            mEngine->connectSource(prefetch, this);
-        }
+    // prefetch the previous and following comic for faster navigation
+    if (mCurrent.hasNext()) {
+        const QString prefetch = mCurrent.id() + QLatin1Char(':') + mCurrent.next();
+        mEngine->requestSource(prefetch);
+    }
+    if (mCurrent.hasPrev()) {
+        const QString prefetch = mCurrent.id() + QLatin1Char(':') + mCurrent.prev();
+        mEngine->requestSource(prefetch);
     }
 
     updateView();
@@ -205,13 +218,6 @@ void ComicApplet::updateView()
     updateContextMenu();
 }
 
-void ComicApplet::loadProviders()
-{
-    if (mEngine) {
-        QMetaObject::invokeMethod(mEngine, "loadProviders");
-    }
-}
-
 void ComicApplet::positionFullView(QWindow *window)
 {
     if (!window || !window->screen()) {
@@ -281,7 +287,7 @@ void ComicApplet::updateUsedComics()
 
     delete mCheckNewStrips;
     mCheckNewStrips = nullptr;
-    if (mEngine && mCheckNewComicStripsInterval) {
+    if (mCheckNewComicStripsInterval) {
         mCheckNewStrips = new CheckNewStrips(mTabIdentifier, mEngine, mCheckNewComicStripsInterval, this);
         connect(mCheckNewStrips, &CheckNewStrips::lastStrip, this, &ComicApplet::slotFoundLastStrip);
     }
@@ -330,10 +336,9 @@ void ComicApplet::configChanged()
     mCheckNewComicStripsInterval = cg.readEntry("checkNewComicStripsIntervall", 30);
 
     auto oldMaxComicLimit = mMaxComicLimit;
-    mMaxComicLimit = cg.readEntry("maxComicLimit", CACHE_LIMIT);
-    if (oldMaxComicLimit != mMaxComicLimit && mEngine) {
-        mEngine->disconnectSource(QLatin1String("setting_maxComicLimit:") + QString::number(oldMaxComicLimit), this);
-        mEngine->connectSource(QLatin1String("setting_maxComicLimit:") + QString::number(mMaxComicLimit), this);
+    mMaxComicLimit = cg.readEntry("maxComicLimit", 29);
+    if (oldMaxComicLimit != mMaxComicLimit) {
+        mEngine->setMaxComicLimit(mMaxComicLimit);
     }
 
     globalComicUpdater->load();
@@ -465,7 +470,7 @@ void ComicApplet::updateComic(const QString &identifierSuffix)
     const QString id = mCurrent.id();
     setConfigurationRequired(id.isEmpty());
 
-    if (!id.isEmpty() && mEngine) {
+    if (!id.isEmpty()) {
         setBusy(true);
 
         const QString identifier = id + QLatin1Char(':') + identifierSuffix;
@@ -474,18 +479,14 @@ void ComicApplet::updateComic(const QString &identifierSuffix)
         // if there was an error only disconnect the oldSource if it had nothing to do with the error or if the comic changed, that way updates of the error can
         // come in
         if (!mIdentifierError.isEmpty() && !mIdentifierError.contains(id)) {
-            mEngine->disconnectSource(mIdentifierError, this);
             mIdentifierError.clear();
         }
-        if ((mIdentifierError != mOldSource) && (mOldSource != identifier)) {
-            mEngine->disconnectSource(mOldSource, this);
-        }
         mOldSource = identifier;
-        mEngine->connectSource(identifier, this);
+        mEngine->requestSource(identifier);
         slotScaleToContent();
     } else {
         qWarning() << "Either no identifier was specified or the engine could not be created:"
-                   << "id" << id << "engine valid:" << mEngine;
+                   << "id" << id;
         setConfigurationRequired(true);
     }
     updateContextMenu();
@@ -495,9 +496,6 @@ void ComicApplet::updateContextMenu()
 {
     if (mCurrent.id().isEmpty()) {
         mActiveComicModel->clear();
-    }
-
-    if (mCurrent.id().isEmpty() || !mCurrent.ready()) {
         mActionNextNewStripTab->setEnabled(false);
         mActionGoFirst->setEnabled(false);
         mActionGoLast->setEnabled(false);
@@ -801,6 +799,6 @@ bool ComicApplet::isTabHighlighted(const QString &id) const
     return false;
 }
 
-K_PLUGIN_CLASS_WITH_JSON(ComicApplet, "package/metadata.json")
+K_PLUGIN_CLASS(ComicApplet)
 
 #include "comic.moc"
