@@ -1,6 +1,7 @@
 /*
  *   SPDX-FileCopyrightText: 2011 Matthias Fuchs <mat69@gmx.net>
  *   SPDX-FileCopyrightText: 2015 Marco Martin <mart@kde.org>
+ *   SPDX-FileCopyrightText: 2022 Alexander Lohnau <alexander.lohnau@gmx.de>
  *
  *   SPDX-License-Identifier: GPL-2.0-or-later
  */
@@ -10,8 +11,8 @@
 
 #include "comicinfo.h"
 
+#include "engine/comic.h"
 #include <KIO/Job>
-#include <Plasma/DataEngine>
 
 class QTemporaryFile;
 class KZip;
@@ -30,7 +31,7 @@ public:
 
     /**
      * Creates a comic archive job.
-     * The engine has to be a working comic dataengine.
+     * The engine has to be a working comic engine.
      * The archiveType defines what kind of input is given, e.g. if ArchiveAll is
      * used no other parameters need to be defined, while ArchiveFromTo needs
      * both toIdentifier and fromIdentifier (from <= to!), the other two types need only the toIdentifier.
@@ -39,7 +40,7 @@ public:
      * @see setToIdentifier, setFromIdentifier
      */
     ComicArchiveJob(const QUrl &dest,
-                    Plasma::DataEngine *engine,
+                    ComicEngine *engine,
                     ArchiveType archiveType,
                     IdentifierType identifierType,
                     const QString &pluginName,
@@ -65,15 +66,13 @@ public:
 
     void start() override;
 
-public Q_SLOTS:
-    void dataUpdated(const QString &source, const Plasma::DataEngine::Data &data);
-
 protected:
     bool doKill() override;
     bool doSuspend() override;
     bool doResume() override;
 
 private:
+    void dataUpdated(const ComicMetaData &data);
     /**
      * Sets the total number of comics to download.
      * @param currentSuffix if empty the from and to identifier suffix will be used.
@@ -119,7 +118,7 @@ private:
     int mComicNumber;
     int mProcessedFiles;
     int mTotalFiles;
-    Plasma::DataEngine *mEngine;
+    ComicEngine *mEngine;
     QTemporaryFile *mZipFile;
     KZip *mZip;
     QString mPluginName;
