@@ -2,6 +2,7 @@
  *   SPDX-FileCopyrightText: 2007 Tobias Koenig <tokoe@kde.org>
  *   SPDX-FileCopyrightText: 2008-2010 Matthias Fuchs <mat69@gmx.net>
  *   SPDX-FileCopyrightText: 2015 Marco Martin <mart@kde.org>
+ *   SPDX-FileCopyrightText: 2022 Alexander Lohnau <alexander.lohnau@gmx.de>
  *
  *   SPDX-License-Identifier: GPL-2.0-or-later
  */
@@ -11,17 +12,14 @@
 
 #include <QAbstractTableModel>
 
-#include <Plasma/DataEngine>
-#include <Plasma/DataEngineConsumer>
+#include "engine/comic.h"
 
-class ComicModel : public QAbstractTableModel, public Plasma::DataEngineConsumer
+class ComicModel : public QAbstractTableModel
 {
     Q_OBJECT
 
 public:
-    ComicModel(Plasma::DataEngine *engine, const QString &source, const QStringList &usedComics, QObject *parent = nullptr);
-
-    void setComics(const Plasma::DataEngine::Data &comics, const QStringList &usedComics);
+    ComicModel(ComicEngine *engine, const QStringList &usedComics, QObject *parent = nullptr);
 
     QHash<int, QByteArray> roleNames() const override;
 
@@ -30,11 +28,8 @@ public:
     QVariant data(const QModelIndex &index, int role = Qt::CheckStateRole) const override;
     Qt::ItemFlags flags(const QModelIndex &index) const override;
 
-public Q_SLOTS:
-    void dataUpdated(const QString &source, const Plasma::DataEngine::Data &data);
-
 private:
-    Plasma::DataEngine::Data mComics;
+    QList<ComicProviderInfo> mComics;
     QStringList mUsedComics;
 };
 
